 ''
'' Program: GameMenu
''
'' Author: Jim Goodpaster as WordFire 
''
'' Note: the program locks the four screens together to keep people "on the same page"
''
'' Jan 7, 2018: added remote control and audio, set num of apps max at 48 (6 pages of 8)
'' Sep 8, 2018: modified to work with Mac OSX text files terminated with only LF, not CRLF
'' Note: the stack is down to 122 longs. Probably shouldn't go any lower 
''
CON                                 
                                                                              
  _clkmode = XTAL1|PLL16X                                                                    
  _xinfreq = 5_000_000

CON                                                        

  'Pin assignments
  IRPin = 25     'for IR remote control receiver   
  VOL = 20       'volume level control pin
  SND = 21       'audio/sound output pin  
  SD_DO = 4      'SD Card DO/MISO
  SD_CK = 5      'SD Card Clock (SCLK)
  SD_DI = 9      'SD Card DI/MOSI  
  SD_CS = 15     'SD Card Chip Select, no pull-up resistor
  'data and clock pin assignments, respectively, in QuadKeyboard driver (in two places)
  'kbd0: 7, 8; kbd1: 12, 13; kbd2: 16, 17; kbd3: 23, 24 

  #0,  NORTH, WEST, SOUTH, EAST, NWSE     '4 scr nums (0..3), counterclockwise from north, plus all 4 screens (4)

  #0, MOUNT, UNMOUNT

  '6 possible pages of 8 apps each for a total of 48 apps on an SD Card (info stored in HUB)
  NOAM = 48 'Number Of Apps Maximum (on SD card)
  NL = 8  'Number of Lines of apps on a page
  LL = 7  'last line of page (NL - 1)

  'foregnd-backgnd color pairs; avoid KK ($00) = Blk-on-Blk (string delimiter); don't use WR wht-on-red (reserved word)
  KK = $00, KR = $01, KG = $02, KB = $03, KY = $04, KP = $05, KA = $06, KW = $07 'blk foreground on some background (Note 1)
  RK = $10, RR = $11, RG = $12, RB = $13, RY = $14, RP = $15, RA = $16, RW = $17 'red foreground on some background
  GK = $20, GR = $21, GG = $22, GB = $23, GY = $24, GP = $25, GA = $26, GW = $27 'grn foreground on some backgroung  
  BK = $30, BR = $31, BG = $32, BB = $33, BY = $34, BP = $35, BA = $36, BW = $37 'blu foreground on some backgroung 
  YK = $40, YR = $41, YG = $42, YB = $43, YY = $44, YP = $45, YA = $46, YW = $47 'yel foreground on some backgroung
  PK = $50, PR = $51, PG = $52, PB = $53, PY = $54, PP = $55, PA = $56, PW = $57 'pur foreground on some backgroung     
  AK = $60, AR = $61, AG = $62, AB = $63, AY = $64, AP = $65, AA = $66, AW = $67 'aqu foreground on some backgroung  
  WK = $70, WS = $71, WG = $72, WB = $73, WY = $74, WP = $75, WA = $76, WW = $77 'wht foreground on some backgroung (Note 2)
  'Note 1: don't use KK ($00) = Blk-on-Blk in strings since 0 is the string delimiter value (for terminating strings) 
  'Note 2: for white-on-red, use white-on-scarlet (the scarlet letter) = WS because WR is a reserved word in PASM 
  'byte    %%0000, %%0010, %%0020, %%1000  ' 0: Black,      1: Red,        2: Green,      3: Blue  'Quaternary form
  'byte    %%0030, %%1010, %%1020, %%1030  ' 4: Yellow,     5: Purple,     6: Aqua        7: White 'Quaternary form
  
OBJ

  vga[4] : "vga"                      'VGA interface to VGA driver
  sdc    : "SD-MMC1A"                 'SD card driver (FAT16), can also reload RAM from SD .bin file    
  kbd    : "QuadKeyboard"             'Quad keyboard driver (JRG)
  ir     : "ir_reader_nec"            'infrared remote control receiver   
  
VAR

  long font[32*32/2]            'for custom (user-defined) font, first 32 characters     
  
  long seed                     'seed for random number generator for random effects

  long rv                       'generic return value
  long sdErrMsg                 'SD card error string
  long sdErrNum                 'SD card error number
  byte sdProblem                'problem starting SD driver or mounting card; true/false
  byte ms[5]                    'valid Menu Selections for SD card
   
  byte kbs                      'keyboard selector (round-robin)
  
  byte d0, d1, d2, d3           'numeric digits printed as characters, one by one

  byte BC, HF                   'background color, header/footer color 

  byte page, pages              'page number of total pages; exp: 1/3 = pg. 1 of 3 pgs
  byte noa                      'number of apps
  byte lch                      'last character (key pressed)
  long row                      'currently selected (highlighted) row
  
  
DAT '                                ###   THE MAIN LOOP   ###
PUB main | s, i, ch, po, ro, can
's=screen, ch=character from keyboard, po = page offset, record offset, current app number

  initSystem     'start up system: video, keyboards, sd card, etc.

  BC := BB       'set background color (e.g., BB or YY)    
  HF := YB       'set header/footer color (e.g., YB or YY)
 
  page := 1
  row := -1      'upon start only, used to show instructions 
  
  repeat s from 0 to 3
    showSplash(s)

  waitcnt(clkfreq*2 + cnt)   'just in case the SD card needs time to initializa/stabilize, etc.
  getAppData                 'read in app names and descriptions into internal dat arrays
  sortAppData                'sort the array of app names
  waitcnt(clkfreq*1 + cnt)   'hold time for splash screen
  showFanfold                'display form with fields in fanfold paper effect
  
  'process keys
  repeat 'main loop, infinite
    'remote control (18=pwr, 27=up, 4=left, 5=center, 6=right, 26=down) 
    if (ch := ir.fifo_get) <> -1   
      kbs := 0
      case ch
        18:    beep(100, 2000) 'power button
        27:    ch := 232  'up
        4:     ch := 228  'left
        5:     ch := 13   'center - launch game/program  
        6:     ch := 230  'right
        26:    ch := 226  'down          
    else
      'receive any pending input from any keyboard, first come, first serve
      kbs := (kbs + 1) // 4    'cycle among the keyboards (0..3, wrapping around)
      ch := kbd.getKey(kbs)    'get key in keyboard buffer, if present 
    
    if noa == 0                      'ignore keyboards if no apps
      next  
    if ch <> 0                       'check if there's a buffered keystroke for curr. kbd.
    
      if ch == 32                    'if key is a space character, then...
        ch := 226                    'map space to down arrow (226) for two cases below
        
      can := (page-1)*NL+row         'calculate current app number
      po := (page-1)*NL              'first record slot of current page
      ro := (po+row)*37              'calculate record offset
      
      if ch == 226 and row == -1     'down arrow after program load, one time only
        row := 0
        ro := (po+row)*37        
        repeat s from 0 to 3     
          showAppNum(s)                  'show app number        
          repChar(s, 2, 2, WW, " ", 36)        
          write(s, 2, 2, KW, @names[ro]) 'show first app entry (in slot zero)
          putChar(s, 1, 2, YW, 5)        'yellow left border on white bgnd  
          putChar(s, 38, 2, YW, 4)       'yellow right border on white bgnd
        showDescription                  'show brief description of app
        
      elseif ch == 232 and row == -1     'up arrow after page load
        'ignore up arrow if down arrow hasn't been used yet to select something
          
      elseif ch == 226 or ch == lch      'down arrow or repeated lowercase letter
        if can+1 => noa                  'surpress passing last record (on filled or unfilled page)
          next
        elseif row == LL                 'advance to next page
          row := 0
          page++
          showFanFold
          next
        repeat s from 0 to 3       
          repChar(s, 2, row+2, LC[row//2], " ", 36)  'erase white selection         
          write(s, 2, row+2, TC[row//2], @names[ro]) 'restore app name
          putChar(s, 1, row+2, FC[row//2], 5)    'yellow left border   
          putChar(s, 38, row+2, FC[row//2], 4)   'yellow right border            
        row := row + 1 <# LL
        ro := (po+row)*37
        repeat s from 0 to 3  
          repChar(s, 2, row+2, WW, " ", 36)      'show selection on white  
          write(s, 2, row+2, KW, @names[ro])     'app name
          putChar(s, 1, row+2, YW, 5)    'yellow left border on white bgnd
          putChar(s, 38, row+2, YW, 4)   'yellow right border on white bgnd
          showAppNum(s)                  'show app number
        showDescription                  'show brief description of app
                      
      elseif ch == 232                   'up arrow
        if row == 0 and page == 1        'ignore if already at top of 1st page
          next
        elseif row == 0                  'go to previous page
          row := LL
          page--
          showFanFold
          next
        repeat s from 0 to 3
          repChar(s, 2, row+2, LC[row//2], " ", 36)  'erase white selection                                                  
          write(s, 2, row+2, TC[row//2], @names[ro]) 'restore app name
          putChar(s, 1, row+2, FC[row//2], 5)    'yellow left border   
          putChar(s, 38, row+2, FC[row//2], 4)   'yellow right border                
        row := row - 1 #> 0
        ro := (po+row)*37        
        repeat s from 0 to 3         
          repChar(s, 2, row+2, WW, " ", 36)      'show selection on white  
          write(s, 2, row+2, KW, @names[ro])     'app name
          putChar(s, 1, row+2, YW, 5)    'yellow left border on white bgnd
          putChar(s, 38, row+2, YW, 4)   'yellow right border on white bgnd
          showAppNum(s)                  'show app number
        showDescription                  'show brief description of app
                              
      elseif ch == 228                   'left arrow
        if page > 1
          page--                         'previous page
          row := 0
          showFanFold
          
      elseif ch == 230                   'right arrow
        if page < pages
          page++                         'next page
          row := 0
          showFanFold
          
      elseif ch => "1" and ch =< "8"     'test if key character is a number
        ch := ch - 48                    'convert char to number 1..8
        if ch > pages                    'ignore key if page doesn't exist
          next                           'return to top of main loop
        if ch == page                    'ignore if key is the current page
          next                           'return to top of main loop
        page := ch                       'change the page number
        row := 0                         'use first row of page
        showFanFold                      'jump to selected page
        
      elseif ch => "a" and ch =< "z"     'use lowercase letter to jump to page
        repeat i from 0 to noa-1         'walk app names to look for nearest match
          if names[i*37]|$20 => ch       'if first byte of app name is => than letter
            page := i/NL + 1             'calc page
            row := i - ((page-1)*NL)     'calc row to highlight on jumping
            showFanFold                  'jump to new page
            lch := ch                    'remember letter; if pressed again, use down arrow (see above)
            quit                         'end processing ch
            
      elseif ch == 13                    'check if ENTER was pressed
        if row => 0                      'make sure an app is selected
          bootSDCApp                     'launch selected app/game, returns only if error

      ifnot (ch => "a" and ch =< "z")    'reset last char (lch) if ch was not a lowercase letter
        lch := "~"                       'unlikely value, not a lowercase letter    

PUB showFanFold | s, i, ro
  repeat s from 0 to 3
    'show header
    if row == -1  'avoids flashing, only clear when needed
      clear(s, BC)                       'clear screen to background color
      repChar(s, 1, 0, HF, 1, 38)        'header top      
      repChar(s, 1, 1, RR, " ", 38)      'header text background
      write(s, 11, 1, WS, string("SD Card Games/Apps"))

    showPageNum(s)                       'show page/pages on the right side of header
    showAppNum(s)                        'show the app num on the left side of header

    'show header
    if row == -1                    
      putChar(s, 1, 1, YR, 5)            'header left bar
      putChar(s, 38, 1, YR, 4)           'header right bar 

    'show "fan-fold paper"
    repeat i from 2 to NL+1
      if i // 2 'non-zero promoted to true
        repChar(s, 1, i, YY, " ", 38)
      else
        repChar(s, 1, i, GG, " ", 38)

    'show left and right borders with yellow
    repeat i from 2 to NL+1
      putChar(s, 1, i, FC[i//2], 5)    'footer left bar    
      putChar(s, 38, i, FC[i//2], 4)   'footer right bar
        
    showAppNames(s)                      'populate rows with app names
 
    'highlight current row, if any    
    if row => 0 and row < NL
      ro := ((page-1)*NL + row) * 37
      repChar(s, 2, row+2, WW, " ", 36)  'make whole row white
      putChar(s, 1, row+2, YW, 5)        'yellow left border on white bgnd  
      putChar(s, 38, row+2, YW, 4)       'yellow right border on white bgnd         
      write(s, 2, row+2, KW, @names[ro]) 'highlight current selection        

    'show footer for descriptions/instructions
    repeat i from 10 to 13   
      repChar(s, 1, i, PP, " ", 38)      'text area  
      putChar(s, 1, i, YP, 5)            'footer left bar
      putChar(s, 38, i, YP, 4)           'footer right bar

    if row == -1                          
      repeat i from 0 to 3               'show instructions or app description  
        write(s, 2, 10+i, WP, @INS[i*37])

      repChar(s, 1, 14, HF, 10, 38)      'footer bottom         

  if row => 0
    showDescription
      
PUB getAppData | i, ptr, ss, descaddr

  'populate name array with names  
  noa := 0  'number of names
  repeat  'read SD card
    sdc.listEntries("N")
    if sdc.listIsDirectory
      path[0] := "/"     
      bytemove(@path+1, sdc.listName, strsize(sdc.listName)) 'append directory name to path
      bytemove(@dirs[noa*9], sdc.listName, 8)  'save app directory name in array of app directories
      
      repeat i from 1 to 8  'find first space char, if any
        if path[i] == " "
          dirs[noa*9+i-1] := 0      'used when booting app from SD card
          quit
      
      path[i] := "/"  'append slash to folder name
      bytemove(@path+i+1, string("ABOUT.TXT"), 9)      
      path[i+11] := 0                                                       
      'try to open About.txt file for folder
      ptr := \sdc.openFile(@path, "R")         'note: closes any open file
      
      if strcomp(ptr, string("ABOUT.TXT   "))  'need to fill w/spaces for 8.3 format       
        'read line of file
        line[0] := 0
        \sdc.readString(@line, 41)
        if (ss := strsize(@line)) > 0
          if line[ss-1] < 32
            line[ss-1] := 0                    'remove possible carriage return
          bytemove(@names[37*noa], @line, 36)  'populate name dat array
          noa++

        repeat i from 0 to 3
          descaddr := (noa-1)*37*4
          line[0] := 0                      
          \sdc.readString(@line, 38)           'get line
          if line[0] == 10                     'if line is only the LF char...
            \sdc.readString(@line, 38)         '...toss it and get another line
          ss := strsize(@line)
          if line[ss-1] < 32
            line[ss-1] := 0                    'remove possible carriage return             
          bytemove(@desc[descaddr+i*37], @line, 36)
          
    else
      if strsize(sdc.listName) == 0
        quit
    if noa => NOAM 'number of apps maximum
      quit
  
  pages := (noa-1)/NL + 1  'calculate the total number of pages of app names, NL apps/pg               
  if noa == 0              'if no directories (or no SD card?)
    pages := 1             'default to 1 page, which will be blank

  sdErrMsg := \sdc.closeFile    
  'shutdown                 'unmount SD card

PUB showDescription | s, i, can, descaddr
's=screen, i=index, can=current app no., descaddr=description addr 
  can := (page-1)*NL+row         'calculate current app number
  descaddr := can*4*37           'calculate address of 1st line of description for current app

  'update app description box
  repeat s from 0 to 3
    repeat i from 10 to 13
      repChar(s, 2, i, PP, " ", 36)                'erase description box
    'show four-line description
    write(s, 2, 10, WP, @desc[descaddr])           'show 1st line of description
    write(s, 2, 11, WP, @desc[descaddr+37])        'show 2nd line of description
    write(s, 2, 12, WP, @desc[descaddr+74])        'show 3rd line of description
    write(s, 2, 13, WP, @desc[descaddr+111])       'show 4th line of description               
   
PUB showAppNames(s) | i, j
  'show list of app names
  if noa == 0
    return

  j := (page-1)*NL                             'starting record/app      
  repeat i from 0 to 7
    if i // 2
      write(s, 2, i+2, BY, @names[j*37])
    else                     
      write(s, 2, i+2, BG, @names[j*37])
    j++                                        'advance to next record

PUB showPageNum(s)
  putChar(s, 35, 1, AR, page+48)  'current page number
  putChar(s, 36, 1, AR, "/")      'slash separator (pg/pages)
  putChar(s, 37, 1, AR, pages+48) 'total number of pages

PUB showAppNum(s)
  if row == -1
    return
  showNN(s, 2, 1, AR, (page-1)*NL+row+1)         

PUB sortAppData | i, p1, p2, p3, p4, p5, p6, didSwap
'i=index to loop through apps, didSwap=boolean to indicate if a swap was performed 
'p1/p2=ptrs to long app names, p3/p4=ptrs to app directories, p5/p6=ptrs to app descriptions 
'Sorts long app names, directory names and descriptions in ascending order based on long app names
'Uses standard (slow) Bubble Sort technique to perform the sort

  repeat
    didSwap := false                             'assume no swap occurred each pass
    repeat i from 0 to (noa-2)                   'loop through apps (noa=number of apps)
      p1 := @names[i*37]                         'set up pointers
      p2 := @names[(i+1)*37]
      p3 := @dirs[i*9]
      p4 := @dirs[(i+1)*9]
      p5 := @desc[i*37*4]
      p6 := @desc[(i+1)*37*4]
                            
      if (strCompare(p1, p2) > 0)                'check if s1 > s2. If so, then...
        bytemove(@temp, p1, strsize(p1)+1)       'swap long app names
        bytemove(p1, p2, strsize(p2)+1)
        bytemove(p2, @temp, strsize(@temp)+1)
        
        bytemove(@temp, p3, 9)                   'swap directory (folders) names
        bytemove(p3, p4, 9)
        bytemove(p4, @temp, 9)

        bytemove(@temp, p5, 148)                 'swap descriptions (4*37=148)
        bytemove(p5, p6, 148)
        bytemove(p6, @temp, 148)                
                                                 'continue sorting as not done yet
        didSwap := true 

  while didSwap
     
PUB strCompare(str1, str2)
'str1/str2=pointers to incoming strings to compare
'determine the alphabetical order of two given strings
'return 0 if the two strings are equal, otherwise...
'return a positive number if s1 > s2, or a negative number if s1 < s2

  repeat
    if (byte[str1] == byte[str2])                'if current characters of two strings are the same
      if (byte[str1] == 0)                       'if at end of string = null character
        return 0                                 'the strings are equal, so return 0 for equality
      else                                       'otherwise the strings are not equal
        str1++                                   'bump both string pointers
        str2++
    else
      quit                                       'quit if two characters not equal and compare
                                                 
  return byte[str1]&$DF - byte[str2]&$DF         'return pos or neg number, ignore letter case

DAT                            'SCREEN MODE 1 = Splash
PUB showSplash(s) | i
'show "Game Time" splash screen
  clear(s, WW) 'clear screen to white

  repeat i from 0 to 4
    write(s, 3, i+2, BW, @SGame[i*35])
    write(s, 3, i+8, RW, @SMenu[i*35])

DAT '                           OUTPUT FUNCTIONS FOR SCREENS
PUB setXYC(s, c, r, k)  'set cursor pos and color
  's = screen, c = column, r = row, k = color
  vga[s].out($A)
  vga[s].out(c)
  vga[s].out($B)
  vga[s].out(r)   
  vga[s].out($1B)         
  vga[s].out("c")
  vga[s].out(k)

PUB putChar(s, c, r, k, ch)  'put (not out) character
  's=screen, r=row, c=column, k=color, ch=character 
  setXYC(s, c, r, k)
  vga[s].putc(ch)

PUB putChar4(c, r, k, ch) | s  'put (not out) character to all screens
  's=screen, r=row, c=column, k=color, ch=character
  repeat s from 0 to 3
    setXYC(s, c, r, k)
    vga[s].putc(ch)
  
PUB write(s, c, r, k, str) 'output string to screen s at r, c in color k
  's=screen, c=column, r=row, k=color, str=string to write  
  vga[s].str2(c, r, k, str)

PUB write2(s, c, r, k, addr, n) 'output string to screen s at r, c in color k
  's=screen, c=column, r=row, k=color, str=string to write  
  vga[s].str3(c, r, k, addr, n)

PUB clear(s, k) | i, beg, end  'clear screen to solid background color
's = screen (0..3 or 4 for all screens), k = fore/background color
  beg := end := s
  if s => 4
    beg := NORTH
    end := EAST

  repeat i from beg to end
    vga[i].clear(k)

PUB repChar(s, c, r, k, ch, rep)  'output a single character severals times in a row
'display character ch from column c to column c+rep-1 (rep = repeat) on row r in color k
  setXYC(s, c, r, k)   
  repeat rep
    vga[s].putc(ch)

PUB showNN(s, c, r, k, val) | i, beg, end  'show a two-digit number on one or four screens
'draw two-digit number on one or all four screens
's=screen or use 4 for all, c=column, r=row, k=color, val=value to display
  beg := end := s
  if s == 4
    beg := 0
    end := 3

  d1 := val / 10
  d0 := val // 10
  repeat i from beg to end   
    putChar(i, c, r, k, d1+48)
    putChar(i, c+1, r, k, d0+48)

DAT '                                ### SD CARD ### 
PUB sdcall(funct)
  case funct
    0: sdc.mountPartition(0) ' Mount the default partition 0. Can be 0 - 3.
    1: sdc.unmountPartition

PUB startSDCardDriver
  'start FAT driver for SD Card
  'do, clk, di, cs, wp, cd, rtcda, rtcclk, rtcbuslck/i2clock  
  rv := sdc.fatEngineStart( SD_DO, SD_CK, SD_DI, SD_CS, -1, -1, -1, -1, -1)
  sdProblem := !rv
  if !rv
    sdErrMsg := string("Driver could not start")
    
PUB mountSDCard
  'mount sd card
  sdProblem := false
  sdErrMsg := \sdcall(MOUNT) 'Returns the address of the error string or null.
  sdErrNum := sdc.partitionError 'Returns the error number or zero.
  if sdErrNum > 0
    sdProblem := true

PUB shutdownSDCard
  sdErrMsg := \sdc.closeFile 'close the open file, if there is one open
  sdErrMsg := \sdcall(UNMOUNT)
  '\sdc.FATEngineStop

PUB bootSDCApp | i, s, can, ss
'i=loop index, s=screen, can=current app number, ss=string size
'note: code relies on directory names already being stripped of trailing spaces (null terminated)
  can := (page-1)*NL+row                   'calculate current app number

  'path guide: "12345678901234567890123456789"    '1-based 
  'path guide: "012345678901234567890123456789"   '0-based
  'path guide: "/12345678/12345678.BIN" + null = 22 + 1 = 23 total chars 

  repeat i from 0 to 23                    'clear path to all null chars
    path[i] := 0
  
  path[0] := "/"                                     'build absolute path
  ss := strsize(@dirs[can*9])                        'get length of app directory name
  bytemove(@path+1, @dirs[can*9], ss)                'append directory name 
  path[strsize(@path)] := "/"                        'append slash to directory name
  bytemove(@path+strsize(@path), @dirs[can*9], ss)   'append file name = directory
  bytemove(@path+strsize(@path), string(".BIN"), 4)  'append file extension (Propeller binary)

  \sdc.bootPartition(@path)  'launch selected app and don't return (replaces this gamemenu app)

  'the following code is executed only if the app didn't launch (see previous line of code)
  repeat s from 0 to 3                                             'show error message
    repeat i from 10 to 13
      repChar(s, 2, i, PP, " ", 36)                                'erase description box  
    write(s, 4, 11, WP, string("The file"))                        
    write(s, 13, 11, YB, @path)      'show path on a blue background to reveal any spaces
    write(s, 4, 12, WP, string("was not found or has a problem."))
 
PUB restartSDC
'restart SD card driver and mount the first partition of the SD card
  \sdc.unmountPartition      'unmount the SD card (if mounted)
  sdc.FATEngineStop          'stop the SD card driver (if running)
  waitcnt(clkfreq/20+cnt)    'give SD card time, just in case
  startSDCardDriver          'try starting the driver again
  mountSDCard                'and try mounting the SD card (if any) 

DAT '                                         UTILITY FUNCTIONS
PUB wait(sec)
  waitcnt(clkfreq*sec+cnt)

PUB waitf(div)
  waitcnt(clkfreq/div+cnt)

PUB beep(d, fr)
'duration (number of wave cycles and relative frequency

  repeat d
    outa[SND]~                     'one wave cycle
    waitcnt(clkfreq/fr+cnt)
    outa[SND]~~
    waitcnt(clkfreq/fr+cnt) 
  
DAT '                          ### INITIALIZATION ###
PUB initSystem | s, i, vpins, lock

  'config audio pins
  dira[SND]~~                   'audio output pin
  dira[VOL]~~                   'volume control pin  

  'set speaker volume
  ctra :=  %00110_000 << 23 + 1 << 9 + VOL   'counter mode = duty cycle for d/a using rc circuit (B pin is unused)
  frqa := $2000_0000                         'Vout = 3.3 * (frqa/2^32) = 3.3 * 0.5 = 1.65V max when frqa = $8000_0000 (can omit this line) 

  'configure video pins
  lock := ((cnt >> 16 + 2) | 1) << 16   
  repeat s from 0 to 3 
    vga[s].init(lock|vconfig[s])                'initialize vid drv with vgrp/vpins       

  repeat s from 0 to 3
    'vga[s].setn(1, @pal0)                       'switch to palette 0   
    vga[s].str(string($1B, "s"))                'page mode
    'vga[s].setn(1, constant(NEGX|$8000))       'use the ROM font for chars 0..31
    vga[s].setn(1, NEGX|@user_font)             'use the custom font for chars 0..31
    waitf(4) 'let voltage to screens stablize?
    
  'start quad keyboard driver
  kbd.start
  kbs := 3 'keyboard selector (round robin), optional init code

  ir.init(IRPin, 0, 10, false)   'start infrared reader for remote control    

  waitf(4)                        'just in case
  startSDCardDriver               'start SDC driver
  waitf(4)                        'just in case
  mountSDCard                     'mount card (assumes it's present)
    
  repeat i from 0 to 3            'defaults, etc.
    kbd.clearKeyBuffer(i)         'empty kbd buffers (probably not strictly necessary)
    
DAT 'vconfig
vconfig long    0 << 9 | %%103_3     'with sync   %010011_11  
        long    1 << 9 | %%103_0     'no sync     %010011_00      
        long    2 << 9 | %%103_0     '   
        long    3 << 9 | %%103_0     '           

DAT

{
  This technique is from Roger Williams' vga_40x18_rom_demo.spin v1.1  
  The user defined chars must be reversed and interleaved like the ROM font,
  .... This example shows a way to draw the font forward and un-interleaved
  so that the proptool will assemble it correctly.  Draw the left/lsb=0
  character with 2's and the right/lsb=1 character with 1's.   
}

user_font 'beginning address of user's custom font data

              'Cust Char 0: bot half      Cust Char 1: bot whole    
C00_01  long  %%00000000_00000000 >< 32 + %%00000000_00000000 >< 32 'used for seconds
        long  %%00000000_00000000 >< 32 + %%00000000_00000000 >< 32 'indicator that
        long  %%00000000_00000000 >< 32 + %%00000000_00000000 >< 32 'marches across
        long  %%00000000_00000000 >< 32 + %%00000000_00000000 >< 32 'Big Clock screen
        long  %%00000000_00000000 >< 32 + %%00000000_00000000 >< 32
        long  %%00000000_00000000 >< 32 + %%00000000_00000000 >< 32
        long  %%00000000_00000000 >< 32 + %%00000000_00000000 >< 32
        long  %%00000000_00000000 >< 32 + %%00000000_00000000 >< 32                                                         
        long  %%00000000_00000000 >< 32 + %%00000000_00000000 >< 32
        long  %%00000000_00000000 >< 32 + %%00000000_00000000 >< 32
        long  %%00000000_00000000 >< 32 + %%00000000_00000000 >< 32
        long  %%00000000_00000000 >< 32 + %%00000000_00000000 >< 32
        long  %%00000000_00000000 >< 32 + %%00000000_00000000 >< 32
        long  %%00000000_00000000 >< 32 + %%00000000_00000000 >< 32
        long  %%00000000_00000000 >< 32 + %%00000000_00000000 >< 32
        long  %%00000000_00000000 >< 32 + %%00000000_00000000 >< 32
        long  %%00000000_00000000 >< 32 + %%00000000_00000000 >< 32
        long  %%00000000_00000000 >< 32 + %%00000000_00000000 >< 32
        long  %%00000000_00000000 >< 32 + %%00000000_00000000 >< 32
        long  %%00000000_00000000 >< 32 + %%00000000_00000000 >< 32
        long  %%00000000_00000000 >< 32 + %%00000000_00000000 >< 32
        long  %%00000000_00000000 >< 32 + %%00000000_00000000 >< 32
        long  %%00000000_00000000 >< 32 + %%00000000_00000000 >< 32
        long  %%00000000_00000000 >< 32 + %%00000000_00000000 >< 32
        long  %%00000000_00000000 >< 32 + %%00000000_00000000 >< 32
        long  %%00000000_00000000 >< 32 + %%00000000_00000000 >< 32
        long  %%00000000_00000000 >< 32 + %%00000000_00000000 >< 32
        long  %%00000000_00000000 >< 32 + %%00000000_00000000 >< 32
        long  %%22222222_00000000 >< 32 + %%11111111_11111111 >< 32
        long  %%22222222_00000000 >< 32 + %%11111111_11111111 >< 32
        long  %%22222222_00000000 >< 32 + %%11111111_11111111 >< 32
        long  %%22222222_00000000 >< 32 + %%11111111_11111111 >< 32


              'Cust Char 2: RH4LS    Cust Char 3: LH4RS = left half for right side
C02_03  long  %%00000000_22222222 >< 32 + %%11111111_00000000 >< 32
        long  %%00000000_22222222 >< 32 + %%11111111_00000000 >< 32
        long  %%00000000_22222222 >< 32 + %%11111111_00000000 >< 32
        long  %%00000000_22222222 >< 32 + %%11111111_00000000 >< 32
        long  %%00000000_22222222 >< 32 + %%11111111_00000000 >< 32
        long  %%00000000_22222222 >< 32 + %%11111111_00000000 >< 32
        long  %%00000000_22222222 >< 32 + %%11111111_00000000 >< 32
        long  %%00000000_22222222 >< 32 + %%11111111_00000000 >< 32
        long  %%00000000_22222222 >< 32 + %%11111111_00000000 >< 32
        long  %%00000000_22222222 >< 32 + %%11111111_00000000 >< 32
        long  %%00000000_22222222 >< 32 + %%11111111_00000000 >< 32
        long  %%00000000_22222222 >< 32 + %%11111111_00000000 >< 32
        long  %%00000000_22222222 >< 32 + %%11111111_00000000 >< 32
        long  %%00000000_22222222 >< 32 + %%11111111_00000000 >< 32
        long  %%00000000_22222222 >< 32 + %%11111111_00000000 >< 32
        long  %%00000000_22222222 >< 32 + %%11111111_00000000 >< 32
        long  %%00000000_22222222 >< 32 + %%11111111_00000000 >< 32
        long  %%00000000_22222222 >< 32 + %%11111111_00000000 >< 32
        long  %%00000000_22222222 >< 32 + %%11111111_00000000 >< 32
        long  %%00000000_22222222 >< 32 + %%11111111_00000000 >< 32
        long  %%00000000_22222222 >< 32 + %%11111111_00000000 >< 32
        long  %%00000000_22222222 >< 32 + %%11111111_00000000 >< 32
        long  %%00000000_22222222 >< 32 + %%11111111_00000000 >< 32
        long  %%00000000_22222222 >< 32 + %%11111111_00000000 >< 32
        long  %%00000000_22222222 >< 32 + %%11111111_00000000 >< 32
        long  %%00000000_22222222 >< 32 + %%11111111_00000000 >< 32
        long  %%00000000_22222222 >< 32 + %%11111111_00000000 >< 32
        long  %%00000000_22222222 >< 32 + %%11111111_00000000 >< 32
        long  %%00000000_22222222 >< 32 + %%11111111_00000000 >< 32
        long  %%00000000_22222222 >< 32 + %%11111111_00000000 >< 32
        long  %%00000000_22222222 >< 32 + %%11111111_00000000 >< 32  
        long  %%00000000_22222222 >< 32 + %%11111111_00000000 >< 32          

              'Cust Char 4: shim4leftside Cust Char 5: shim4rightside        
C04_05  long  %%00000000_00002222 >< 32 + %%11110000_00000000 >< 32
        long  %%00000000_00002222 >< 32 + %%11110000_00000000 >< 32
        long  %%00000000_00002222 >< 32 + %%11110000_00000000 >< 32
        long  %%00000000_00002222 >< 32 + %%11110000_00000000 >< 32
        long  %%00000000_00002222 >< 32 + %%11110000_00000000 >< 32
        long  %%00000000_00002222 >< 32 + %%11110000_00000000 >< 32
        long  %%00000000_00002222 >< 32 + %%11110000_00000000 >< 32
        long  %%00000000_00002222 >< 32 + %%11110000_00000000 >< 32                                                         
        long  %%00000000_00002222 >< 32 + %%11110000_00000000 >< 32
        long  %%00000000_00002222 >< 32 + %%11110000_00000000 >< 32
        long  %%00000000_00002222 >< 32 + %%11110000_00000000 >< 32
        long  %%00000000_00002222 >< 32 + %%11110000_00000000 >< 32
        long  %%00000000_00002222 >< 32 + %%11110000_00000000 >< 32
        long  %%00000000_00002222 >< 32 + %%11110000_00000000 >< 32
        long  %%00000000_00002222 >< 32 + %%11110000_00000000 >< 32
        long  %%00000000_00002222 >< 32 + %%11110000_00000000 >< 32
        long  %%00000000_00002222 >< 32 + %%11110000_00000000 >< 32
        long  %%00000000_00002222 >< 32 + %%11110000_00000000 >< 32
        long  %%00000000_00002222 >< 32 + %%11110000_00000000 >< 32
        long  %%00000000_00002222 >< 32 + %%11110000_00000000 >< 32
        long  %%00000000_00002222 >< 32 + %%11110000_00000000 >< 32
        long  %%00000000_00002222 >< 32 + %%11110000_00000000 >< 32
        long  %%00000000_00002222 >< 32 + %%11110000_00000000 >< 32
        long  %%00000000_00002222 >< 32 + %%11110000_00000000 >< 32
        long  %%00000000_00002222 >< 32 + %%11110000_00000000 >< 32
        long  %%00000000_00002222 >< 32 + %%11110000_00000000 >< 32
        long  %%00000000_00002222 >< 32 + %%11110000_00000000 >< 32
        long  %%00000000_00002222 >< 32 + %%11110000_00000000 >< 32
        long  %%00000000_00002222 >< 32 + %%11110000_00000000 >< 32
        long  %%00000000_00002222 >< 32 + %%11110000_00000000 >< 32
        long  %%00000000_00002222 >< 32 + %%11110000_00000000 >< 32
        long  %%00000000_00002222 >< 32 + %%11110000_00000000 >< 32                 

              'Cust Char 6: Diag1         Cust Char 7: Diag2          
C06_07  long  %%00000000_00000000 >< 32 + %%00000000_00000000 >< 32  'Chars C06_07 & C08_09
        long  %%00000000_00000002 >< 32 + %%10000000_00000000 >< 32  'are in the order of
        long  %%00000000_00000002 >< 32 + %%10000000_00000000 >< 32  'the corners of an 'O'
        long  %%00000000_00000022 >< 32 + %%11000000_00000000 >< 32  'clockwise from the
        long  %%00000000_00000022 >< 32 + %%11000000_00000000 >< 32  'upper-left corner of
        long  %%00000000_00000222 >< 32 + %%11100000_00000000 >< 32  'the letter 'O' to the.
        long  %%00000000_00000222 >< 32 + %%11100000_00000000 >< 32  'lower-left corner.                                                       
        long  %%00000000_00002222 >< 32 + %%11110000_00000000 >< 32
        long  %%00000000_00002222 >< 32 + %%11110000_00000000 >< 32
        long  %%00000000_00022222 >< 32 + %%11111000_00000000 >< 32
        long  %%00000000_00022222 >< 32 + %%11111000_00000000 >< 32
        long  %%00000000_00222222 >< 32 + %%11111100_00000000 >< 32
        long  %%00000000_00222222 >< 32 + %%11111100_00000000 >< 32
        long  %%00000000_02222222 >< 32 + %%11111110_00000000 >< 32
        long  %%00000000_02222222 >< 32 + %%11111110_00000000 >< 32
        long  %%00000000_22222222 >< 32 + %%11111111_00000000 >< 32
        long  %%00000000_22222222 >< 32 + %%11111111_00000000 >< 32
        long  %%00000002_22222222 >< 32 + %%11111111_10000000 >< 32
        long  %%00000002_22222222 >< 32 + %%11111111_10000000 >< 32
        long  %%00000022_22222222 >< 32 + %%11111111_11000000 >< 32
        long  %%00000022_22222222 >< 32 + %%11111111_11000000 >< 32
        long  %%00000222_22222222 >< 32 + %%11111111_11100000 >< 32
        long  %%00000222_22222222 >< 32 + %%11111111_11100000 >< 32
        long  %%00002222_22222222 >< 32 + %%11111111_11110000 >< 32
        long  %%00002222_22222222 >< 32 + %%11111111_11110000 >< 32
        long  %%00022222_22222222 >< 32 + %%11111111_11111000 >< 32
        long  %%00022222_22222222 >< 32 + %%11111111_11111000 >< 32
        long  %%00222222_22222222 >< 32 + %%11111111_11111100 >< 32
        long  %%00222222_22222222 >< 32 + %%11111111_11111100 >< 32
        long  %%02222222_22222222 >< 32 + %%11111111_11111110 >< 32
        long  %%02222222_22222222 >< 32 + %%11111111_11111110 >< 32
        long  %%22222222_22222222 >< 32 + %%11111111_11111111 >< 32          

              'Cust Char 8: Diag3         Cust Char 9: Diag4         
C08_09  long  %%22222222_22222222 >< 32 + %%11111111_11111111 >< 32  
        long  %%22222222_22222220 >< 32 + %%01111111_11111111 >< 32
        long  %%22222222_22222220 >< 32 + %%01111111_11111111 >< 32
        long  %%22222222_22222200 >< 32 + %%00111111_11111111 >< 32
        long  %%22222222_22222200 >< 32 + %%00111111_11111111 >< 32
        long  %%22222222_22222000 >< 32 + %%00011111_11111111 >< 32
        long  %%22222222_22222000 >< 32 + %%00011111_11111111 >< 32                                                         
        long  %%22222222_22220000 >< 32 + %%00001111_11111111 >< 32
        long  %%22222222_22220000 >< 32 + %%00001111_11111111 >< 32
        long  %%22222222_22200000 >< 32 + %%00000111_11111111 >< 32
        long  %%22222222_22200000 >< 32 + %%00000111_11111111 >< 32
        long  %%22222222_22000000 >< 32 + %%00000011_11111111 >< 32
        long  %%22222222_22000000 >< 32 + %%00000011_11111111 >< 32
        long  %%22222222_20000000 >< 32 + %%00000001_11111111 >< 32
        long  %%22222222_20000000 >< 32 + %%00000001_11111111 >< 32
        long  %%22222222_00000000 >< 32 + %%00000000_11111111 >< 32
        long  %%22222222_00000000 >< 32 + %%00000000_11111111 >< 32
        long  %%22222220_00000000 >< 32 + %%00000000_01111111 >< 32
        long  %%22222220_00000000 >< 32 + %%00000000_01111111 >< 32
        long  %%22222200_00000000 >< 32 + %%00000000_00111111 >< 32
        long  %%22222200_00000000 >< 32 + %%00000000_00111111 >< 32
        long  %%22222000_00000000 >< 32 + %%00000000_00011111 >< 32
        long  %%22222000_00000000 >< 32 + %%00000000_00011111 >< 32
        long  %%22220000_00000000 >< 32 + %%00000000_00001111 >< 32
        long  %%22220000_00000000 >< 32 + %%00000000_00001111 >< 32
        long  %%22200000_00000000 >< 32 + %%00000000_00000111 >< 32
        long  %%22200000_00000000 >< 32 + %%00000000_00000111 >< 32
        long  %%22000000_00000000 >< 32 + %%00000000_00000011 >< 32
        long  %%22000000_00000000 >< 32 + %%00000000_00000011 >< 32
        long  %%20000000_00000000 >< 32 + %%00000000_00000001 >< 32
        long  %%20000000_00000000 >< 32 + %%00000000_00000001 >< 32
        long  %%00000000_00000000 >< 32 + %%00000000_00000000 >< 32 

              'Cust Char 10: Block         Cust Char 11: thin line         
C10_11  long  %%22222222_22222222 >< 32 + %%11111111_11111111 >< 32  
        long  %%22222222_22222222 >< 32 + %%11111111_11111111 >< 32
        long  %%22222222_22222222 >< 32 + %%11111111_11111111 >< 32
        long  %%22222222_22222222 >< 32 + %%11111111_11111111 >< 32
        long  %%00000000_00000000 >< 32 + %%00000000_00000000 >< 32
        long  %%00000000_00000000 >< 32 + %%00000000_00000000 >< 32
        long  %%00000000_00000000 >< 32 + %%00000000_00000000 >< 32                                                         
        long  %%00000000_00000000 >< 32 + %%00000000_00000000 >< 32
        long  %%00000000_00000000 >< 32 + %%00000000_00000000 >< 32
        long  %%00000000_00000000 >< 32 + %%00000000_00000000 >< 32
        long  %%00000000_00000000 >< 32 + %%00000000_00000000 >< 32
        long  %%00000000_00000000 >< 32 + %%00000000_00000000 >< 32
        long  %%00000000_00000000 >< 32 + %%00000000_00000000 >< 32
        long  %%00000000_00000000 >< 32 + %%00000000_00000000 >< 32
        long  %%00000000_00000000 >< 32 + %%00000000_00000000 >< 32
        long  %%00000000_00000000 >< 32 + %%00000000_00000000 >< 32
        long  %%00000000_00000000 >< 32 + %%00000000_00000000 >< 32
        long  %%00000000_00000000 >< 32 + %%00000000_00000000 >< 32
        long  %%00000000_00000000 >< 32 + %%00000000_00000000 >< 32
        long  %%00000000_00000000 >< 32 + %%00000000_00000000 >< 32
        long  %%00000000_00000000 >< 32 + %%00000000_00000000 >< 32
        long  %%00000000_00000000 >< 32 + %%00000000_00000000 >< 32
        long  %%00000000_00000000 >< 32 + %%00000000_00000000 >< 32
        long  %%00000000_00000000 >< 32 + %%00000000_00000000 >< 32
        long  %%00000000_00000000 >< 32 + %%00000000_00000000 >< 32
        long  %%00000000_00000000 >< 32 + %%00000000_00000000 >< 32
        long  %%00000000_00000000 >< 32 + %%00000000_00000000 >< 32
        long  %%00000000_00000000 >< 32 + %%00000000_00000000 >< 32
        long  %%00000000_00000000 >< 32 + %%11111111_11111111 >< 32
        long  %%00000000_00000000 >< 32 + %%11111111_11111111 >< 32
        long  %%00000000_00000000 >< 32 + %%11111111_11111111 >< 32
        long  %%00000000_00000000 >< 32 + %%11111111_11111111 >< 32

              'Cust Char 12: bot line     Cust Char 13: thin line         
C12_13  long  %%00000000_00000000 >< 32 + %%11111111_11111111 >< 32  
        long  %%00000000_00000000 >< 32 + %%11111111_11111111 >< 32
        long  %%00000000_00000000 >< 32 + %%11111111_11111111 >< 32
        long  %%00000000_00000000 >< 32 + %%11111111_11111111 >< 32
        long  %%00000000_00000000 >< 32 + %%11111111_11111111 >< 32
        long  %%00000000_00000000 >< 32 + %%11111111_11111111 >< 32
        long  %%00000000_00000000 >< 32 + %%11111111_11111111 >< 32                                                         
        long  %%00000000_00000000 >< 32 + %%11111111_11111111 >< 32
        long  %%00000000_00000000 >< 32 + %%11111111_11111111 >< 32
        long  %%00000000_00000000 >< 32 + %%11111111_11111111 >< 32
        long  %%00000000_00000000 >< 32 + %%00000000_00000000 >< 32
        long  %%00000000_00000000 >< 32 + %%00000000_00000000 >< 32
        long  %%00000000_00000000 >< 32 + %%00000000_00000000 >< 32
        long  %%00000000_00000000 >< 32 + %%00000000_00000000 >< 32
        long  %%00000000_00000000 >< 32 + %%00000000_00000000 >< 32
        long  %%00000000_00000000 >< 32 + %%00000000_00000000 >< 32
        long  %%00000000_00000000 >< 32 + %%00000000_00000000 >< 32
        long  %%00000000_00000000 >< 32 + %%00000000_00000000 >< 32
        long  %%00000000_00000000 >< 32 + %%00000000_00000000 >< 32
        long  %%00000000_00000000 >< 32 + %%00000000_00000000 >< 32
        long  %%00000000_00000000 >< 32 + %%00000000_00000000 >< 32
        long  %%00000000_00000000 >< 32 + %%00000000_00000000 >< 32
        long  %%22222222_22222222 >< 32 + %%00000000_00000000 >< 32
        long  %%22222222_22222222 >< 32 + %%00000000_00000000 >< 32
        long  %%22222222_22222222 >< 32 + %%00000000_00000000 >< 32
        long  %%22222222_22222222 >< 32 + %%00000000_00000000 >< 32
        long  %%22222222_22222222 >< 32 + %%00000000_00000000 >< 32
        long  %%22222222_22222222 >< 32 + %%00000000_00000000 >< 32
        long  %%22222222_22222222 >< 32 + %%00000000_00000000 >< 32
        long  %%22222222_22222222 >< 32 + %%00000000_00000000 >< 32
        long  %%22222222_22222222 >< 32 + %%00000000_00000000 >< 32
        long  %%22222222_22222222 >< 32 + %%00000000_00000000 >< 32 

              'Cust Char 14: Full Block   Cust Char 15: ellipsis...         
C14_15  long  %%22222222_22222222 >< 32 + %%00000000_00000000 >< 32  
        long  %%22222222_22222222 >< 32 + %%00000000_00000000 >< 32
        long  %%22222222_22222222 >< 32 + %%00000000_00000000 >< 32
        long  %%22222222_22222222 >< 32 + %%00000000_00000000 >< 32
        long  %%22222222_22222222 >< 32 + %%00000000_00000000 >< 32
        long  %%22222222_22222222 >< 32 + %%00000000_00000000 >< 32
        long  %%22222222_22222222 >< 32 + %%00000000_00000000 >< 32                                                         
        long  %%22222222_22222222 >< 32 + %%00000000_00000000 >< 32
        long  %%22222222_22222222 >< 32 + %%00000000_00000000 >< 32
        long  %%22222222_22222222 >< 32 + %%00000000_00000000 >< 32
        long  %%22222222_22222222 >< 32 + %%00000000_00000000 >< 32
        long  %%22222222_22222222 >< 32 + %%00000000_00000000 >< 32
        long  %%22222222_22222222 >< 32 + %%00000000_00000000 >< 32
        long  %%22222222_22222222 >< 32 + %%00000000_00000000 >< 32
        long  %%22222222_22222222 >< 32 + %%00000000_00000000 >< 32
        long  %%22222222_22222222 >< 32 + %%00000000_00000000 >< 32
        long  %%22222222_22222222 >< 32 + %%00000000_00000000 >< 32
        long  %%22222222_22222222 >< 32 + %%00000000_00000000 >< 32
        long  %%22222222_22222222 >< 32 + %%00000000_00000000 >< 32
        long  %%22222222_22222222 >< 32 + %%00000000_00000000 >< 32
        long  %%22222222_22222222 >< 32 + %%00000000_00000000 >< 32
        long  %%22222222_22222222 >< 32 + %%00000000_00000000 >< 32
        long  %%22222222_22222222 >< 32 + %%00000000_00000000 >< 32
        long  %%22222222_22222222 >< 32 + %%00111001_11001110 >< 32
        long  %%22222222_22222222 >< 32 + %%00111001_11001110 >< 32
        long  %%22222222_22222222 >< 32 + %%00111001_11001110 >< 32
        long  %%22222222_22222222 >< 32 + %%00111001_11001110 >< 32
        long  %%22222222_22222222 >< 32 + %%00000000_00000000 >< 32
        long  %%22222222_22222222 >< 32 + %%00000000_00000000 >< 32
        long  %%22222222_22222222 >< 32 + %%00000000_00000000 >< 32
        long  %%22222222_22222222 >< 32 + %%00000000_00000000 >< 32
        long  %%22222222_22222222 >< 32 + %%00000000_00000000 >< 32


              'Cust Char X: Full Block 2  Cust Char X: VU Indicator  
C16_17  long  %%00000000_00000000 >< 32 + %%00000000_00000000 >< 32
        long  %%00000000_00000000 >< 32 + %%00000000_00000000 >< 32
        long  %%00000000_00000000 >< 32 + %%00000000_00000000 >< 32
        long  %%00000000_00000000 >< 32 + %%00000000_00000000 >< 32                   
        long  %%00000000_00000000 >< 32 + %%00000000_00000000 >< 32
        long  %%00000000_00000000 >< 32 + %%00000000_00000000 >< 32
        long  %%00000002_20000000 >< 32 + %%00000001_10000000 >< 32
        long  %%00000002_20000000 >< 32 + %%00000001_10000000 >< 32
        long  %%00000022_22000000 >< 32 + %%00000001_10000000 >< 32
        long  %%00000022_22000000 >< 32 + %%00000001_10000000 >< 32                                                         
        long  %%00000222_22200000 >< 32 + %%00000001_10000000 >< 32
        long  %%00000222_22200000 >< 32 + %%00000001_10000000 >< 32
        long  %%00002202_20220000 >< 32 + %%00000001_10000000 >< 32
        long  %%00002202_20220000 >< 32 + %%00000001_10000000 >< 32
        long  %%00022002_20022000 >< 32 + %%00000001_10000000 >< 32
        long  %%00022002_20022000 >< 32 + %%00110001_10001100 >< 32
        long  %%00220002_20002200 >< 32 + %%00110001_10001100 >< 32
        long  %%00220002_20002200 >< 32 + %%00011001_10011000 >< 32
        long  %%00000002_20000000 >< 32 + %%00011001_10011000 >< 32
        long  %%00000002_20000000 >< 32 + %%00001101_10110000 >< 32
        long  %%00000002_20000000 >< 32 + %%00001101_10110000 >< 32
        long  %%00000002_20000000 >< 32 + %%00000111_11100000 >< 32
        long  %%00000002_20000000 >< 32 + %%00000111_11100000 >< 32
        long  %%00000002_20000000 >< 32 + %%00000011_11000000 >< 32
        long  %%00000002_20000000 >< 32 + %%00000011_11000000 >< 32
        long  %%00000002_20000000 >< 32 + %%00000001_10000000 >< 32
        long  %%00000002_20000000 >< 32 + %%00000001_10000000 >< 32
        long  %%00000000_00000000 >< 32 + %%00000000_00000000 >< 32
        long  %%00000000_00000000 >< 32 + %%00000000_00000000 >< 32
        long  %%00000000_00000000 >< 32 + %%00000000_00000000 >< 32
        long  %%00000000_00000000 >< 32 + %%00000000_00000000 >< 32
        long  %%00000000_00000000 >< 32 + %%00000000_00000000 >< 32
        long  %%00000000_00000000 >< 32 + %%00000000_00000000 >< 32         

DAT

SGame         byte 06,14,14,14,14,14,14,07, 32, 06,14,14,14,14,14,07, 32, 14,14,07,32,32,32,32,06,14,14, 32, 14,14,14,14,14,03, 0 '35
              byte 14,14,32,32,32,32,32,32, 32, 14,14,32,32,32,14,14, 32, 14,14,14,07,32,32,06,14,14,14, 32, 14,14,32,32,32,32, 0
              byte 14,14,32,32,14,14,14,14, 32, 14,14,14,14,14,14,14, 32, 14,14,09,14,07,06,14,08,14,14, 32, 14,14,14,14,32,32, 0
              byte 14,14,32,32,32,32,14,14, 32, 14,14,32,32,32,14,14, 32, 14,14,32,09,14,14,08,32,14,14, 32, 14,14,32,32,32,32, 0
              byte 09,14,14,14,14,14,14,08, 32, 14,14,32,32,32,14,14, 32, 14,14,32,32,09,08,32,32,14,14, 32, 14,14,14,14,14,03, 0 
              
SMenu         byte 14,14,07,32,32,32,32,06,14,14, 32, 14,14,14,14,14,14,14, 32, 14,14,07,32,32,14,14, 32, 14,14,32,32,32,14,14, 0 '35
              byte 14,14,14,07,32,32,06,14,14,14, 32, 14,14,32,32,32,32,32, 32, 14,14,14,07,32,14,14, 32, 14,14,32,32,32,14,14, 0
              byte 14,14,09,14,07,06,14,08,14,14, 32, 14,14,14,14,14,32,32, 32, 14,14,09,14,07,14,14, 32, 14,14,32,32,32,14,14, 0
              byte 14,14,32,09,14,14,08,32,14,14, 32, 14,14,32,32,32,32,32, 32, 14,14,32,09,14,14,14, 32, 14,14,32,32,32,14,14, 0
              byte 14,14,32,32,09,08,32,32,14,14, 32, 14,14,14,14,14,14,14, 32, 14,14,32,32,09,14,14, 32, 09,14,14,14,14,14,08, 0 
      
DAT
 
path          byte 0[24]       'directory and file name (8.3 format), 23 + 1 for good measure
line          byte 0[41]       'line from file on SD card  '37?

temp          byte 0[150]       'temp buffer for swapping/sorting strings, etc
names         byte 0[NOAM*37]    'app name  s: 50 names x (36+1=37) bytes per name
dirs          byte 0[NOAM*9]     'directory names (with about.txt files) off SD card
desc          byte 0[NOAM*4*37]  'four-line descriptions for fifty apps/games


colors        byte KB, KY, KR, KG, BK, YK, RK, GK, WB, KY, WS, KG, BW, YW, RW, GW
cards         byte BW, YW, RW, GW 'for NWSE white cards for cardinal directions
TP            byte WK, WS, KG, WB, KY, KA, WP, KW  'test pattern colors              
PFC           byte KG, WS 'pass-fail colors for SD/RTC checks

KBC           byte YY, BY, BB, KY, YK, WK, KY, RR, RY, WS  'color sets for big clock
              byte BB, GB, GG, WB, BW, BW, WB, RR, RB, WS  '  10 bytes per set
              byte WW, RW, RR, KW, WK, WK, KW, BB, BW, YB  '  4 sets
              byte KK, YK, YY, WK, KB, WS, RK, BB, BK, WB  '  backgrounds: YY, BB, WW, KK

LC            byte GG, YY 'blank line color for "fan-fold paper" effect
TC            byte BG, BY 'text color ofr "fan-fold paper" effect
FC            byte YG, YY 'for left and right borders of "fan-fold" area
             
INS           byte "Use the ",16," ",17," keys to pick an app.    ",0
              byte "Use the ",186," ",187," keys for last/next page.",0  '37
              byte "Use 1-8 or a-z to jump among pages. ",0
              byte "Press ENTER to run an app/game.",0 'length of last string doesn't matter

DAT           'end of file